@@ -47,9 +47,6 @@ import org.apache.tomcat.util.security.ConcurrentMessageDigest;
 /**
  * An <b>Authenticator</b> and <b>Valve</b> implementation of HTTP DIGEST Authentication, as outlined in RFC 7616: "HTTP
  * Digest Authentication"
- *
- * @author Craig R. McClanahan
- * @author Remy Maucherat
  */
 public class DigestAuthenticator extends AuthenticatorBase {
 
@@ -68,7 +65,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
     private static final String NONCE_DIGEST = "SHA-256";
 
     // List permitted algorithms and maps them to Java standard names
-    private static final Map<String, AuthDigest> PERMITTED_ALGORITHMS = new HashMap<>();
+    private static final Map<String,AuthDigest> PERMITTED_ALGORITHMS = new HashMap<>();
     static {
         // Allows the digester to be configured with either the Standard Java name or the name used the RFC.
         for (AuthDigest authDigest : AuthDigest.values()) {
@@ -91,7 +88,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
     /**
      * List of server nonce values currently being tracked
      */
-    protected Map<String, NonceInfo> nonces;
+    protected Map<String,NonceInfo> nonces;
 
 
     /**
@@ -208,7 +205,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
 
     public String getAlgorithms() {
         StringBuilder result = new StringBuilder();
-        StringUtils.join(algorithms, ',', (x) -> x.getRfcName(), result);
+        StringUtils.join(algorithms, ',', AuthDigest::getRfcName, result);
         return result.toString();
     }
 
@@ -233,7 +230,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
 
 
     /*
-     *  Initialise algorithms, removing ones that the JRE does not support
+     * Initialise algorithms, removing ones that the JRE does not support
      */
     private void initAlgorithms(List<AuthDigest> algorithms) {
         Iterator<AuthDigest> algorithmIterator = algorithms.iterator();
@@ -243,7 +240,8 @@ public class DigestAuthenticator extends AuthenticatorBase {
                 ConcurrentMessageDigest.init(algorithm.getJavaName());
             } catch (NoSuchAlgorithmException e) {
                 // In theory, a JRE can choose not to implement SHA-512/256
-                log.warn(sm.getString("digestAuthenticator.unsupportedAlgorithm", algorithms, algorithm.getJavaName()), e);
+                log.warn(sm.getString("digestAuthenticator.unsupportedAlgorithm", algorithms, algorithm.getJavaName()),
+                        e);
                 algorithmIterator.remove();
             }
         }
@@ -323,7 +321,10 @@ public class DigestAuthenticator extends AuthenticatorBase {
      * @param quotesRequired <code>true</code> if quotes were required
      *
      * @return The unquoted string
+     *
+     * @deprecated This will be removed in Tomcat 11 onwards. Unused.
      */
+    @Deprecated
     protected static String removeQuotes(String quotedString, boolean quotesRequired) {
         // support both quoted and non-quoted
         if (quotedString.length() > 0 && quotedString.charAt(0) != '"' && !quotesRequired) {
@@ -341,7 +342,10 @@ public class DigestAuthenticator extends AuthenticatorBase {
      * @param quotedString The quoted string
      *
      * @return The unquoted string
+     *
+     * @deprecated This will be removed in Tomcat 11 onwards. Unused.
      */
+    @Deprecated
     protected static String removeQuotes(String quotedString) {
         return removeQuotes(quotedString, false);
     }
@@ -368,7 +372,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
 
         String ipTimeKey = request.getRemoteAddr() + ":" + currentTime + ":" + getKey();
 
-        // Note: The digest used to generate the nonce is independent of the the digest used for authentication.
+        // Note: The digest used to generate the nonce is independent of the digest used for authentication.
         byte[] buffer = ConcurrentMessageDigest.digest(NONCE_DIGEST, ipTimeKey.getBytes(StandardCharsets.ISO_8859_1));
         String nonce = currentTime + ":" + HexUtils.toHexString(buffer);
 
@@ -436,7 +440,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
     // ------------------------------------------------------- Lifecycle Methods
 
     @Override
-    protected synchronized void startInternal() throws LifecycleException {
+    protected void startInternal() throws LifecycleException {
         super.startInternal();
 
         // Generate a random secret key
@@ -461,7 +465,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
             private long lastLog = 0;
 
             @Override
-            protected boolean removeEldestEntry(Map.Entry<String, NonceInfo> eldest) {
+            protected boolean removeEldestEntry(Map.Entry<String,NonceInfo> eldest) {
                 // This is called from a sync so keep it simple
                 long currentTime = System.currentTimeMillis();
                 if (size() > getNonceCacheSize()) {
@@ -490,8 +494,8 @@ public class DigestAuthenticator extends AuthenticatorBase {
         private final String opaque;
         private final long nonceValidity;
         private final String key;
-        private final Map<String, NonceInfo> nonces;
-        private boolean validateUri = true;
+        private final Map<String,NonceInfo> nonces;
+        private final boolean validateUri;
 
         private String userName = null;
         private String method = null;
@@ -508,7 +512,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
         private AuthDigest algorithm = null;
 
 
-        public DigestInfo(String opaque, long nonceValidity, String key, Map<String, NonceInfo> nonces,
+        public DigestInfo(String opaque, long nonceValidity, String key, Map<String,NonceInfo> nonces,
                 boolean validateUri) {
             this.opaque = opaque;
             this.nonceValidity = nonceValidity;
@@ -529,10 +533,10 @@ public class DigestAuthenticator extends AuthenticatorBase {
                 return false;
             }
 
-            Map<String, String> directives;
+            Map<String,String> directives;
             try {
                 directives = Authorization.parseAuthorizationDigest(new StringReader(authorization));
-            } catch (IOException e) {
+            } catch (IOException ioe) {
                 return false;
             }
 
@@ -590,7 +594,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
                         absolute.append("://");
                         absolute.append(host);
                         absolute.append(uriQuery);
-                        if (!uri.equals(absolute.toString())) {
+                        if (!uri.contentEquals(absolute)) {
                             return false;
                         }
                     } else {
@@ -630,9 +634,9 @@ public class DigestAuthenticator extends AuthenticatorBase {
                 }
             }
             String serverIpTimeKey = request.getRemoteAddr() + ":" + nonceTime + ":" + key;
-            // Note: The digest used to generate the nonce is independent of the the digest used for authentication/
-            byte[] buffer = ConcurrentMessageDigest.digest(NONCE_DIGEST,
-                    serverIpTimeKey.getBytes(StandardCharsets.ISO_8859_1));
+            // Note: The digest used to generate the nonce is independent of the digest used for authentication/
+            byte[] buffer =
+                    ConcurrentMessageDigest.digest(NONCE_DIGEST, serverIpTimeKey.getBytes(StandardCharsets.ISO_8859_1));
             String digestServerIpTimeKey = HexUtils.toHexString(buffer);
             if (!digestServerIpTimeKey.equals(digestclientIpTimeKey)) {
                 return false;
@@ -680,11 +684,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
             }
 
             // Validate algorithm is one of the algorithms configured for the authenticator
-            if (!algorithms.contains(algorithm)) {
-                return false;
-            }
-
-            return true;
+            return algorithms.contains(algorithm);
         }
 
         public boolean isNonceStale() {
@@ -698,15 +698,15 @@ public class DigestAuthenticator extends AuthenticatorBase {
                     ConcurrentMessageDigest.digest(algorithm.getJavaName(), a2.getBytes(StandardCharsets.ISO_8859_1));
             String digestA2 = HexUtils.toHexString(buffer);
 
-            return realm.authenticate(
-                    userName, response, nonce, nc, cnonce, qop, realmName, digestA2, algorithm.getJavaName());
+            return realm.authenticate(userName, response, nonce, nc, cnonce, qop, realmName, digestA2,
+                    algorithm.getJavaName());
         }
 
     }
 
     public static class NonceInfo {
         private final long timestamp;
-        private final boolean seen[];
+        private final boolean[] seen;
         private final int offset;
         private int count = 0;
 
