@@ -33,13 +33,15 @@ import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.res.StringManager;
 
 /**
- * Implementation of <code>LifecycleListener</code> that will init and and destroy APR.
+ * Implementation of <code>LifecycleListener</code> that will init and destroy APR.
  * <p>
  * This listener must only be nested within {@link Server} elements.
  * <p>
- * <strong>Note</strong>: If you are running Tomcat in an embedded fashion and have more than one Server instance per
- * JVM, this listener <em>must not</em> be added to the {@code Server} instances, but handled outside by the calling
- * code which is bootstrapping the embedded Tomcat instances. Not doing so will lead to JVM crashes.
+ * Only one instance of the APR/Native library may be loaded per JVM. Loading multiple instances will trigger a JVM
+ * crash - typically when the Connectors are destroyed. This listener utilises reference counting to ensure that only
+ * one instance of the APR/Native library is loaded at any one time.
+ * <p>
+ * If multiple listener configurations are found, only the first one initialised will be used.
  *
  * @since 4.1
  */
@@ -99,6 +101,11 @@ public class AprLifecycleListener implements LifecycleListener {
 
     protected static final Object lock = new Object();
 
+    // Guarded by lock
+    private static int referenceCount = 0;
+    private boolean instanceInitialized = false;
+
+
     public static boolean isAprAvailable() {
         // https://bz.apache.org/bugzilla/show_bug.cgi?id=48613
         if (AprStatus.isInstanceCreated()) {
@@ -125,9 +132,14 @@ public class AprLifecycleListener implements LifecycleListener {
 
         if (Lifecycle.BEFORE_INIT_EVENT.equals(event.getType())) {
             synchronized (lock) {
+                instanceInitialized = true;
                 if (!(event.getLifecycle() instanceof Server)) {
                     log.warn(sm.getString("listener.notServer", event.getLifecycle().getClass().getSimpleName()));
                 }
+                if (referenceCount++ != 0) {
+                    // Already loaded (note test is performed before reference count is incremented)
+                    return;
+                }
                 init();
                 for (String msg : initInfoLogMessages) {
                     log.info(msg);
@@ -137,31 +149,38 @@ public class AprLifecycleListener implements LifecycleListener {
                     try {
                         initializeSSL();
                     } catch (Throwable t) {
-                        t = ExceptionUtils.unwrapInvocationTargetException(t);
-                        ExceptionUtils.handleThrowable(t);
-                        log.error(sm.getString("aprListener.sslInit"), t);
+                        Throwable throwable = ExceptionUtils.unwrapInvocationTargetException(t);
+                        ExceptionUtils.handleThrowable(throwable);
+                        log.error(sm.getString("aprListener.sslInit"), throwable);
                     }
                 }
                 // Failure to initialize FIPS mode is fatal
                 if (!(null == FIPSMode || "off".equalsIgnoreCase(FIPSMode)) && !isFIPSModeActive()) {
-                    String errorMessage = sm.getString("aprListener.initializeFIPSFailed");
-                    Error e = new Error(errorMessage);
+                    Error e = new Error(sm.getString("aprListener.initializeFIPSFailed"));
                     // Log here, because thrown error might be not logged
-                    log.fatal(errorMessage, e);
+                    log.fatal(e.getMessage(), e);
                     throw e;
                 }
             }
         } else if (Lifecycle.AFTER_DESTROY_EVENT.equals(event.getType())) {
             synchronized (lock) {
+                // Instance may get destroyed without ever being initialized
+                if (instanceInitialized) {
+                    referenceCount--;
+                }
+                if (referenceCount != 0) {
+                    // Still being used
+                    return;
+                }
                 if (!AprStatus.isAprAvailable()) {
                     return;
                 }
                 try {
                     terminateAPR();
                 } catch (Throwable t) {
-                    t = ExceptionUtils.unwrapInvocationTargetException(t);
-                    ExceptionUtils.handleThrowable(t);
-                    log.info(sm.getString("aprListener.aprDestroy"));
+                    Throwable throwable = ExceptionUtils.unwrapInvocationTargetException(t);
+                    ExceptionUtils.handleThrowable(throwable);
+                    log.warn(sm.getString("aprListener.aprDestroy"), throwable);
                 }
             }
         }
@@ -173,6 +192,8 @@ public class AprLifecycleListener implements LifecycleListener {
         AprStatus.setAprAvailable(false);
         fipsModeActive = false;
         sslInitialized = false; // terminate() will clean the pool
+        // There could be unreferenced SSL_CTX still waiting for GC
+        System.gc();
         Library.terminate();
     }
 
@@ -201,9 +222,9 @@ public class AprLifecycleListener implements LifecycleListener {
             return;
         } catch (Throwable t) {
             // Library present but failed to load
-            t = ExceptionUtils.unwrapInvocationTargetException(t);
-            ExceptionUtils.handleThrowable(t);
-            log.warn(sm.getString("aprListener.aprInitError", t.getMessage()), t);
+            Throwable throwable = ExceptionUtils.unwrapInvocationTargetException(t);
+            ExceptionUtils.handleThrowable(throwable);
+            log.warn(sm.getString("aprListener.aprInitError", throwable.getMessage()), throwable);
             return;
         }
         if (tcnMajor > 1 && "off".equalsIgnoreCase(SSLEngine)) {
@@ -212,8 +233,8 @@ public class AprLifecycleListener implements LifecycleListener {
                 // Tomcat Native 2.x onwards requires SSL
                 terminateAPR();
             } catch (Throwable t) {
-                t = ExceptionUtils.unwrapInvocationTargetException(t);
-                ExceptionUtils.handleThrowable(t);
+                Throwable throwable = ExceptionUtils.unwrapInvocationTargetException(t);
+                ExceptionUtils.handleThrowable(throwable);
             }
             return;
         }
@@ -225,8 +246,8 @@ public class AprLifecycleListener implements LifecycleListener {
                 // is below required.
                 terminateAPR();
             } catch (Throwable t) {
-                t = ExceptionUtils.unwrapInvocationTargetException(t);
-                ExceptionUtils.handleThrowable(t);
+                Throwable throwable = ExceptionUtils.unwrapInvocationTargetException(t);
+                ExceptionUtils.handleThrowable(throwable);
             }
             return;
         }
@@ -254,9 +275,9 @@ public class AprLifecycleListener implements LifecycleListener {
         sslInitialized = true;
 
         String methodName = "randSet";
-        Class<?> paramTypes[] = new Class[1];
+        Class<?>[] paramTypes = new Class[1];
         paramTypes[0] = String.class;
-        Object paramValues[] = new Object[1];
+        Object[] paramValues = new Object[1];
         paramValues[0] = SSLRandomSeed;
         Class<?> clazz = Class.forName("org.apache.tomcat.jni.SSL");
         Method method = clazz.getMethod(methodName, paramTypes);
@@ -268,6 +289,7 @@ public class AprLifecycleListener implements LifecycleListener {
         method = clazz.getMethod(methodName, paramTypes);
         method.invoke(null, paramValues);
 
+        AprStatus.setOpenSSLVersion(SSL.version());
         // OpenSSL 3 onwards uses providers
         boolean usingProviders = tcnMajor > 1 || (tcnVersion > 1233 && (SSL.version() & 0xF0000000L) > 0x20000000);
 
