@@ -18,13 +18,8 @@ package org.apache.catalina.core;
 
 public class Constants {
 
-    /**
-     * Unused. Will be removed in Apache Tomcat 10.1.x onwards.
-     */
-    @Deprecated
-    public static final String Package = "org.apache.catalina.core";
-    public static final int MAJOR_VERSION = 3;
-    public static final int MINOR_VERSION = 1;
+    public static final int MAJOR_VERSION = 6;
+    public static final int MINOR_VERSION = 0;
 
     public static final String JSP_SERVLET_CLASS = "org.apache.jasper.servlet.JspServlet";
 }
