@@ -35,15 +35,16 @@ import java.util.Map;
 import java.util.Properties;
 import java.util.Set;
 
-import javax.annotation.PostConstruct;
-import javax.annotation.PreDestroy;
-import javax.annotation.Resource;
-import javax.ejb.EJB;
 import javax.naming.Context;
 import javax.naming.NamingException;
-import javax.persistence.PersistenceContext;
-import javax.persistence.PersistenceUnit;
-import javax.xml.ws.WebServiceRef;
+
+import jakarta.annotation.PostConstruct;
+import jakarta.annotation.PreDestroy;
+import jakarta.annotation.Resource;
+import jakarta.ejb.EJB;
+import jakarta.persistence.PersistenceContext;
+import jakarta.persistence.PersistenceUnit;
+import jakarta.xml.ws.WebServiceRef;
 
 import org.apache.catalina.ContainerServlet;
 import org.apache.catalina.Globals;
@@ -72,7 +73,7 @@ public class DefaultInstanceManager implements InstanceManager {
     static {
         Class<?> clazz = null;
         try {
-            clazz = Class.forName("javax.ejb.EJB");
+            clazz = Class.forName("jakarta.ejb.EJB");
         } catch (ClassNotFoundException cnfe) {
             // Expected
         }
@@ -80,7 +81,7 @@ public class DefaultInstanceManager implements InstanceManager {
 
         clazz = null;
         try {
-            clazz = Class.forName("javax.persistence.PersistenceContext");
+            clazz = Class.forName("jakarta.persistence.PersistenceContext");
         } catch (ClassNotFoundException cnfe) {
             // Expected
         }
@@ -88,7 +89,7 @@ public class DefaultInstanceManager implements InstanceManager {
 
         clazz = null;
         try {
-            clazz = Class.forName("javax.xml.ws.WebServiceRef");
+            clazz = Class.forName("jakarta.xml.ws.WebServiceRef");
         } catch (ClassNotFoundException cnfe) {
             // Expected
         }
@@ -216,7 +217,8 @@ public class DefaultInstanceManager implements InstanceManager {
                 // This will always return a new Method instance
                 // Making this instance accessible does not affect other instances
                 Method postConstruct = getMethod(clazz, entry);
-                postConstruct.setAccessible(true);
+                // If this doesn't work, just let invoke() fail
+                postConstruct.trySetAccessible();
                 postConstruct.invoke(instance);
             }
         }
@@ -251,7 +253,8 @@ public class DefaultInstanceManager implements InstanceManager {
                 // This will always return a new Method instance
                 // Making this instance accessible does not affect other instances
                 Method preDestroy = getMethod(clazz, entry);
-                preDestroy.setAccessible(true);
+                // If this doesn't work, just let invoke() fail
+                preDestroy.trySetAccessible();
                 preDestroy.invoke(instance);
             }
         }
@@ -526,7 +529,8 @@ public class DefaultInstanceManager implements InstanceManager {
 
         // This will always be a new Field instance
         // Making this instance accessible does not affect other instances
-        field.setAccessible(true);
+        // If this doesn't work, just let set() fail
+        field.trySetAccessible();
         field.set(instance, lookedupResource);
     }
 
@@ -562,7 +566,8 @@ public class DefaultInstanceManager implements InstanceManager {
 
         // This will always be a new Method instance
         // Making this instance accessible does not affect other instances
-        method.setAccessible(true);
+        // If this doesn't work, just let invoke() fail
+        method.trySetAccessible();
         method.invoke(instance, lookedupResource);
     }
 
