@@ -46,11 +46,266 @@ import org.apache.juli.logging.Log;
 /**
  * Implementation of the {@link org.apache.catalina.Store Store} interface that stores serialized session objects in a
  * database. Sessions that are saved are still subject to being expired based on inactivity.
- *
- * @author Bip Thelin
  */
-@SuppressWarnings("deprecation")
-public class DataSourceStore extends JDBCStore {
+public class DataSourceStore extends StoreBase {
+
+    /**
+     * Context name associated with this Store
+     */
+    private String name = null;
+
+    /**
+     * Name to register for this Store, used for logging.
+     */
+    protected static final String storeName = "dataSourceStore";
+
+    /**
+     * name of the JNDI resource
+     */
+    protected String dataSourceName = null;
+
+    /**
+     * Context local datasource.
+     */
+    private boolean localDataSource = false;
+
+    /**
+     * DataSource to use
+     */
+    protected DataSource dataSource = null;
+
+
+    // ------------------------------------------------------------ Table & cols
+
+    /**
+     * Table to use.
+     */
+    protected String sessionTable = "tomcat$sessions";
+
+    /**
+     * Column to use for /Engine/Host/Context name
+     */
+    protected String sessionAppCol = "app";
+
+    /**
+     * Id column to use.
+     */
+    protected String sessionIdCol = "id";
+
+    /**
+     * Data column to use.
+     */
+    protected String sessionDataCol = "data";
+
+    /**
+     * {@code Is Valid} column to use.
+     */
+    protected String sessionValidCol = "valid";
+
+    /**
+     * Max Inactive column to use.
+     */
+    protected String sessionMaxInactiveCol = "maxinactive";
+
+    /**
+     * Last Accessed column to use.
+     */
+    protected String sessionLastAccessedCol = "lastaccess";
+
+    // -------------------------------------------------------------- Properties
+
+    /**
+     * @return the name for this instance (built from container name)
+     */
+    public String getName() {
+        if (name == null) {
+            Container container = manager.getContext();
+            String contextName = container.getName();
+            if (!contextName.startsWith("/")) {
+                contextName = "/" + contextName;
+            }
+            String hostName = "";
+            String engineName = "";
+
+            if (container.getParent() != null) {
+                Container host = container.getParent();
+                hostName = host.getName();
+                if (host.getParent() != null) {
+                    engineName = host.getParent().getName();
+                }
+            }
+            name = "/" + engineName + "/" + hostName + contextName;
+        }
+        return name;
+    }
+
+    @Override
+    public String getStoreName() {
+        return storeName;
+    }
+
+    /**
+     * Set the table for this Store.
+     *
+     * @param sessionTable The new table
+     */
+    public void setSessionTable(String sessionTable) {
+        String oldSessionTable = this.sessionTable;
+        this.sessionTable = sessionTable;
+        support.firePropertyChange("sessionTable", oldSessionTable, this.sessionTable);
+    }
+
+    /**
+     * @return the table for this Store.
+     */
+    public String getSessionTable() {
+        return sessionTable;
+    }
+
+    /**
+     * Set the App column for the table.
+     *
+     * @param sessionAppCol the column name
+     */
+    public void setSessionAppCol(String sessionAppCol) {
+        String oldSessionAppCol = this.sessionAppCol;
+        this.sessionAppCol = sessionAppCol;
+        support.firePropertyChange("sessionAppCol", oldSessionAppCol, this.sessionAppCol);
+    }
+
+    /**
+     * @return the web application name column for the table.
+     */
+    public String getSessionAppCol() {
+        return this.sessionAppCol;
+    }
+
+    /**
+     * Set the Id column for the table.
+     *
+     * @param sessionIdCol the column name
+     */
+    public void setSessionIdCol(String sessionIdCol) {
+        String oldSessionIdCol = this.sessionIdCol;
+        this.sessionIdCol = sessionIdCol;
+        support.firePropertyChange("sessionIdCol", oldSessionIdCol, this.sessionIdCol);
+    }
+
+    /**
+     * @return the Id column for the table.
+     */
+    public String getSessionIdCol() {
+        return this.sessionIdCol;
+    }
+
+    /**
+     * Set the Data column for the table
+     *
+     * @param sessionDataCol the column name
+     */
+    public void setSessionDataCol(String sessionDataCol) {
+        String oldSessionDataCol = this.sessionDataCol;
+        this.sessionDataCol = sessionDataCol;
+        support.firePropertyChange("sessionDataCol", oldSessionDataCol, this.sessionDataCol);
+    }
+
+    /**
+     * @return the data column for the table
+     */
+    public String getSessionDataCol() {
+        return this.sessionDataCol;
+    }
+
+    /**
+     * Set the {@code Is Valid} column for the table
+     *
+     * @param sessionValidCol The column name
+     */
+    public void setSessionValidCol(String sessionValidCol) {
+        String oldSessionValidCol = this.sessionValidCol;
+        this.sessionValidCol = sessionValidCol;
+        support.firePropertyChange("sessionValidCol", oldSessionValidCol, this.sessionValidCol);
+    }
+
+    /**
+     * @return the {@code Is Valid} column
+     */
+    public String getSessionValidCol() {
+        return this.sessionValidCol;
+    }
+
+    /**
+     * Set the {@code Max Inactive} column for the table
+     *
+     * @param sessionMaxInactiveCol The column name
+     */
+    public void setSessionMaxInactiveCol(String sessionMaxInactiveCol) {
+        String oldSessionMaxInactiveCol = this.sessionMaxInactiveCol;
+        this.sessionMaxInactiveCol = sessionMaxInactiveCol;
+        support.firePropertyChange("sessionMaxInactiveCol", oldSessionMaxInactiveCol, this.sessionMaxInactiveCol);
+    }
+
+    /**
+     * @return the {@code Max Inactive} column
+     */
+    public String getSessionMaxInactiveCol() {
+        return this.sessionMaxInactiveCol;
+    }
+
+    /**
+     * Set the {@code Last Accessed} column for the table
+     *
+     * @param sessionLastAccessedCol The column name
+     */
+    public void setSessionLastAccessedCol(String sessionLastAccessedCol) {
+        String oldSessionLastAccessedCol = this.sessionLastAccessedCol;
+        this.sessionLastAccessedCol = sessionLastAccessedCol;
+        support.firePropertyChange("sessionLastAccessedCol", oldSessionLastAccessedCol, this.sessionLastAccessedCol);
+    }
+
+    /**
+     * @return the {@code Last Accessed} column
+     */
+    public String getSessionLastAccessedCol() {
+        return this.sessionLastAccessedCol;
+    }
+
+    /**
+     * Set the JNDI name of a DataSource-factory to use for db access
+     *
+     * @param dataSourceName The JNDI name of the DataSource-factory
+     */
+    public void setDataSourceName(String dataSourceName) {
+        if (dataSourceName == null || dataSourceName.trim().isEmpty()) {
+            manager.getContext().getLogger().warn(sm.getString(getStoreName() + ".missingDataSourceName"));
+            return;
+        }
+        this.dataSourceName = dataSourceName;
+    }
+
+    /**
+     * @return the name of the JNDI DataSource-factory
+     */
+    public String getDataSourceName() {
+        return this.dataSourceName;
+    }
+
+    /**
+     * @return if the datasource will be looked up in the webapp JNDI Context.
+     */
+    public boolean getLocalDataSource() {
+        return localDataSource;
+    }
+
+    /**
+     * Set to {@code true} to cause the datasource to be looked up in the webapp JNDI Context.
+     *
+     * @param localDataSource the new flag value
+     */
+    public void setLocalDataSource(boolean localDataSource) {
+        this.localDataSource = localDataSource;
+    }
+
 
     // --------------------------------------------------------- Public Methods
 
@@ -106,7 +361,7 @@ public class DataSourceStore extends JDBCStore {
                     }
                 }
             } catch (SQLException e) {
-                manager.getContext().getLogger().error(sm.getString(getStoreName() + ".SQLException"), e);
+                manager.getContext().getLogger().error(sm.getString("dataSourceStore.SQLException"), e);
                 keys = new String[0];
                 // Close the connection so that it gets reopened next time
             } finally {
@@ -140,7 +395,7 @@ public class DataSourceStore extends JDBCStore {
                     numberOfTries = 0;
                 }
             } catch (SQLException e) {
-                manager.getContext().getLogger().error(sm.getString(getStoreName() + ".SQLException"), e);
+                manager.getContext().getLogger().error(sm.getString("dataSourceStore.SQLException"), e);
             } finally {
                 release(_conn);
             }
@@ -173,7 +428,7 @@ public class DataSourceStore extends JDBCStore {
                     if (rst.next()) {
                         try (ObjectInputStream ois = getObjectInputStream(rst.getBinaryStream(2))) {
                             if (contextLog.isTraceEnabled()) {
-                                contextLog.trace(sm.getString(getStoreName() + ".loading", id, sessionTable));
+                                contextLog.trace(sm.getString("dataSourceStore.loading", id, sessionTable));
                             }
 
                             _session = (StandardSession) manager.createEmptySession();
@@ -181,13 +436,13 @@ public class DataSourceStore extends JDBCStore {
                             _session.setManager(manager);
                         }
                     } else if (context.getLogger().isDebugEnabled()) {
-                        contextLog.debug(getStoreName() + ": No persisted data object found");
+                        contextLog.debug(sm.getString("dataSourceStore.noObject", id));
                     }
                     // Break out after the finally block
                     numberOfTries = 0;
                 }
             } catch (SQLException e) {
-                contextLog.error(sm.getString(getStoreName() + ".SQLException"), e);
+                contextLog.error(sm.getString("dataSourceStore.SQLException"), e);
             } finally {
                 context.unbind(Globals.IS_SECURITY_ENABLED, oldThreadContextCL);
                 release(_conn);
@@ -213,7 +468,7 @@ public class DataSourceStore extends JDBCStore {
                 // Break out after the finally block
                 numberOfTries = 0;
             } catch (SQLException e) {
-                manager.getContext().getLogger().error(sm.getString(getStoreName() + ".SQLException"), e);
+                manager.getContext().getLogger().error(sm.getString("dataSourceStore.SQLException"), e);
             } finally {
                 release(_conn);
             }
@@ -221,7 +476,7 @@ public class DataSourceStore extends JDBCStore {
         }
 
         if (manager.getContext().getLogger().isTraceEnabled()) {
-            manager.getContext().getLogger().trace(sm.getString(getStoreName() + ".removing", id, sessionTable));
+            manager.getContext().getLogger().trace(sm.getString("dataSourceStore.removing", id, sessionTable));
         }
     }
 
@@ -261,7 +516,7 @@ public class DataSourceStore extends JDBCStore {
                 // Break out after the finally block
                 numberOfTries = 0;
             } catch (SQLException e) {
-                manager.getContext().getLogger().error(sm.getString(getStoreName() + ".SQLException"), e);
+                manager.getContext().getLogger().error(sm.getString("dataSourceStore.SQLException"), e);
             } finally {
                 release(_conn);
             }
@@ -307,8 +562,8 @@ public class DataSourceStore extends JDBCStore {
                         numberOfTries = 0;
                     }
                 } catch (SQLException e) {
-                    manager.getContext().getLogger().error(sm.getString(getStoreName() + ".SQLException"), e);
-                } catch (IOException ignore) {
+                    manager.getContext().getLogger().error(sm.getString("dataSourceStore.SQLException"), e);
+                } catch (IOException ioe) {
                     // Ignore
                 } finally {
                     release(_conn);
@@ -319,13 +574,37 @@ public class DataSourceStore extends JDBCStore {
 
         if (manager.getContext().getLogger().isTraceEnabled()) {
             manager.getContext().getLogger()
-                    .trace(sm.getString(getStoreName() + ".saving", session.getIdInternal(), sessionTable));
+                    .trace(sm.getString("dataSourceStore.saving", session.getIdInternal(), sessionTable));
         }
     }
 
 
     // --------------------------------------------------------- Protected Methods
 
+    /**
+     * Check the connection associated with this store, if it's <code>null</code> or closed try to reopen it. Returns
+     * <code>null</code> if the connection could not be established.
+     *
+     * @return <code>Connection</code> if the connection succeeded
+     */
+    protected Connection getConnection() {
+        Connection conn = null;
+        try {
+            conn = open();
+            if (conn == null || conn.isClosed()) {
+                manager.getContext().getLogger().info(sm.getString("dataSourceStore.checkConnectionDBClosed"));
+                conn = open();
+                if (conn == null || conn.isClosed()) {
+                    manager.getContext().getLogger().info(sm.getString("dataSourceStore.checkConnectionDBReOpenFail"));
+                }
+            }
+        } catch (SQLException ex) {
+            manager.getContext().getLogger().error(sm.getString("dataSourceStore.checkConnectionSQLException"), ex);
+        }
+
+        return conn;
+    }
+
     /**
      * Open (if necessary) and return a database connection for use by this Store.
      *
@@ -333,7 +612,6 @@ public class DataSourceStore extends JDBCStore {
      *
      * @exception SQLException if a database error occurs
      */
-    @Override
     protected Connection open() throws SQLException {
         if (dataSourceName != null && dataSource == null) {
             org.apache.catalina.Context context = getManager().getContext();
@@ -377,7 +655,7 @@ public class DataSourceStore extends JDBCStore {
         if (dataSource != null) {
             return dataSource.getConnection();
         } else {
-            throw new IllegalStateException(sm.getString(getStoreName() + ".missingDataSource"));
+            throw new IllegalStateException(sm.getString("dataSourceStore.missingDataSource"));
         }
     }
 
@@ -386,7 +664,6 @@ public class DataSourceStore extends JDBCStore {
      *
      * @param dbConnection The connection to be closed
      */
-    @Override
     protected void close(Connection dbConnection) {
 
         // Do nothing if the database connection is already closed
@@ -400,14 +677,25 @@ public class DataSourceStore extends JDBCStore {
                 dbConnection.commit();
             }
         } catch (SQLException e) {
-            manager.getContext().getLogger().error(sm.getString(getStoreName() + ".commitSQLException"), e);
+            manager.getContext().getLogger().error(sm.getString("dataSourceStore.commitSQLException"), e);
         }
 
         // Close this database connection, and log any errors
         try {
             dbConnection.close();
         } catch (SQLException e) {
-            manager.getContext().getLogger().error(sm.getString(getStoreName() + ".close"), e); // Just log it here
+            manager.getContext().getLogger().error(sm.getString("dataSourceStore.close"), e);
+        }
+    }
+
+    /**
+     * Release the connection, if it is associated with a connection pool.
+     *
+     * @param conn The connection to be released
+     */
+    protected void release(Connection conn) {
+        if (dataSource != null) {
+            close(conn);
         }
     }
 
