@@ -22,12 +22,13 @@ import java.io.IOException;
 import java.security.AccessController;
 import java.util.Collection;
 
-import javax.servlet.jsp.tagext.TagFileInfo;
-import javax.servlet.jsp.tagext.TagInfo;
-import javax.servlet.jsp.tagext.TagLibraryInfo;
 import javax.xml.parsers.SAXParser;
 import javax.xml.parsers.SAXParserFactory;
 
+import jakarta.servlet.jsp.tagext.TagFileInfo;
+import jakarta.servlet.jsp.tagext.TagInfo;
+import jakarta.servlet.jsp.tagext.TagLibraryInfo;
+
 import org.apache.jasper.Constants;
 import org.apache.jasper.JasperException;
 import org.apache.jasper.JspCompilationContext;
@@ -49,11 +50,7 @@ import org.xml.sax.helpers.AttributesImpl;
 
 /**
  * Class implementing a parser for a JSP document, that is, a JSP page in XML syntax.
- *
- * @author Jan Luehe
- * @author Kin-man Chung
  */
-
 class JspDocumentParser extends DefaultHandler2 implements TagConstants {
 
     private static final String LEXICAL_HANDLER_PROPERTY = "http://xml.org/sax/properties/lexical-handler";
@@ -145,7 +142,8 @@ class JspDocumentParser extends DefaultHandler2 implements TagConstants {
         try {
 
             // Create dummy root and initialize it with given page encodings
-            Node.Root dummyRoot = new Node.Root(null, parent, true);
+            Node.Root dummyRoot = new Node.Root(null, parent, true,
+                    pc.getJspCompilationContext().getOptions().getTempVariableNamePrefix());
             dummyRoot.setPageEncoding(pageEnc);
             dummyRoot.setJspConfigPageEncoding(jspConfigPageEnc);
             dummyRoot.setIsEncodingSpecifiedInProlog(isEncodingSpecifiedInProlog);
