@@ -36,12 +36,7 @@ import org.xml.sax.helpers.AttributesImpl;
 /**
  * This class implements a parser for a JSP page (non-xml view). JSP page grammar is included here for reference. The
  * token '#' that appears in the production indicates the current input token location in the production.
- *
- * @author Kin-man Chung
- * @author Shawn Bayern
- * @author Mark Roth
  */
-
 class Parser implements TagConstants {
 
     private final ParserController parserController;
@@ -136,8 +131,7 @@ class Parser implements TagConstants {
             parser.addInclude(root, pageInfo.getIncludeCoda());
         }
 
-        Node.Nodes page = new Node.Nodes(root);
-        return page;
+        return new Node.Nodes(root);
     }
 
     /**
@@ -244,15 +238,12 @@ class Parser implements TagConstants {
         char ch = (char) reader.peekChar();
         if (Character.isLetter(ch) || ch == '_' || ch == ':') {
             StringBuilder buf = new StringBuilder();
-            buf.append(ch);
-            reader.nextChar();
-            ch = (char) reader.peekChar();
-            while (Character.isLetter(ch) || Character.isDigit(ch) || ch == '.' || ch == '_' || ch == '-' ||
-                    ch == ':') {
+            do {
                 buf.append(ch);
                 reader.nextChar();
                 ch = (char) reader.peekChar();
-            }
+            } while (Character.isLetter(ch) || Character.isDigit(ch) || ch == '.' || ch == '_' || ch == '-' ||
+                    ch == ':');
             return buf.toString();
         }
         return null;
@@ -334,8 +325,8 @@ class Parser implements TagConstants {
             }
         } catch (FileNotFoundException ex) {
             err.jspError(start, "jsp.error.file.not.found", file);
-        } catch (Exception ex) {
-            err.jspError(start, ex.getMessage());
+        } catch (Exception e) {
+            err.jspError(start, e.getMessage());
         }
     }
 
@@ -568,7 +559,7 @@ class Parser implements TagConstants {
     }
 
     /*
-     * Parses a attribute directive with the following syntax: AttributeDirective ::= ( S Attribute)*
+     * Parses an attribute directive with the following syntax: AttributeDirective ::= ( S Attribute)*
      */
     private void parseAttributeDirective(Node parent) throws JasperException {
         Attributes attrs = parseAttributes();
@@ -964,8 +955,7 @@ class Parser implements TagConstants {
         boolean result = false;
 
         if (reader.matchesOptionalSpacesFollowedBy("<jsp:attribute")) {
-            // May be an EmptyBody, depending on whether
-            // There's a "<jsp:body" before the ETag
+            // It may be an EmptyBody, depending on whether there's a "<jsp:body" before the ETag
 
             // First, parse <jsp:attribute> elements:
             parseNamedAttributes(parent);
@@ -1192,7 +1182,7 @@ class Parser implements TagConstants {
         // 'CustomActionJSPContent', or 'CustomActionScriptlessContent'.
         // depending on body-content in TLD.
 
-        // Looking for a body, it still can be empty; but if there is a
+        // Looking for a body, it still can be empty; but if there is
         // a tag body, its syntax would be dependent on the type of
         // body content declared in the TLD.
         String bc;
@@ -1202,7 +1192,7 @@ class Parser implements TagConstants {
             bc = tagFileInfo.getTagInfo().getBodyContent();
         }
 
-        Node tagNode = null;
+        Node tagNode;
         if (tagInfo != null) {
             tagNode = new Node.CustomTag(tagName, prefix, shortTagName, uri, attrs, start, parent, tagInfo,
                     tagHandlerClass);
