@@ -63,9 +63,6 @@ import org.apache.tomcat.util.threads.VirtualThreadExecutor;
 /**
  * @param <S> The type used by the socket wrapper associated with this endpoint. Might be the same as U.
  * @param <U> The type of the underlying socket used by this endpoint. Might be the same as S.
- *
- * @author Mladen Turk
- * @author Remy Maucherat
  */
 public abstract class AbstractEndpoint<S, U> {
 
@@ -1108,8 +1105,8 @@ public abstract class AbstractEndpoint<S, U> {
             } else {
                 return IntrospectionUtils.setProperty(this, name, value, false);
             }
-        } catch (Exception x) {
-            getLog().error(sm.getString("endpoint.setAttributeError", name, value), x);
+        } catch (Exception e) {
+            getLog().error(sm.getString("endpoint.setAttributeError", name, value), e);
             return false;
         }
     }
@@ -1232,7 +1229,9 @@ public abstract class AbstractEndpoint<S, U> {
         try {
             localAddress = getLocalAddress();
         } catch (IOException ioe) {
-            getLog().debug(sm.getString("endpoint.debug.unlock.localFail", getName()), ioe);
+            if (getLog().isDebugEnabled()) {
+                getLog().debug(sm.getString("endpoint.debug.unlock.localFail", getName()), ioe);
+            }
         }
         if (localAddress == null) {
             getLog().warn(sm.getString("endpoint.debug.unlock.localNone", getName()));
