@@ -297,7 +297,7 @@ public class ConnectionPool {
         JdbcInterceptor handler = con.getHandler();
         if (handler==null) {
             if (jmxPool != null) {
-                con.createMBean();
+              con.createMBean();
             }
             //build the proxy handler
             handler = new ProxyConnection(this,con,getPoolProperties().isUseEquals());
@@ -362,12 +362,21 @@ public class ConnectionPool {
      * @return constructor used to instantiate the wrapper object
      * @throws NoSuchMethodException Failed to get a constructor
      */
+    /*
+     * Neither the class nor the constructor are exposed outside of jdbc-pool.
+     * Given the comments in the jdbc-pool code regarding caching for
+     * performance, continue to use Proxy.getProxyClass(). This will need to be
+     * revisited if that method is marked for removal.
+     */
+    @SuppressWarnings("deprecation")
     public Constructor<?> getProxyConstructor(boolean xa) throws NoSuchMethodException {
         //cache the constructor
         if (proxyClassConstructor == null ) {
             Class<?> proxyClass = xa ?
-                Proxy.getProxyClass(ConnectionPool.class.getClassLoader(), new Class[] {Connection.class, PooledConnection.class, XAConnection.class}) :
-                Proxy.getProxyClass(ConnectionPool.class.getClassLoader(), new Class[] {Connection.class, PooledConnection.class});
+                    Proxy.getProxyClass(ConnectionPool.class.getClassLoader(),
+                            new Class[] {Connection.class, javax.sql.PooledConnection.class, XAConnection.class}) :
+                    Proxy.getProxyClass(ConnectionPool.class.getClassLoader(),
+                            new Class[] {Connection.class, javax.sql.PooledConnection.class});
             proxyClassConstructor = proxyClass.getConstructor(new Class[] { InvocationHandler.class });
         }
         return proxyClassConstructor;
@@ -382,7 +391,7 @@ public class ConnectionPool {
     protected void close(boolean force) {
         //are we already closed
         if (this.closed) {
-            return;
+          return;
         }
         //prevent other threads from entering
         this.closed = true;
@@ -401,9 +410,9 @@ public class ConnectionPool {
                 while (con != null) {
                     //close the connection
                     if (pool==idle) {
-                        release(con);
+                      release(con);
                     } else {
-                        abandon(con);
+                      abandon(con);
                     }
                     if (!pool.isEmpty()) {
                         con = pool.poll(1000, TimeUnit.MILLISECONDS);
@@ -417,11 +426,11 @@ public class ConnectionPool {
                 }
             }
             if (pool.isEmpty() && force && pool!=busy) {
-                pool = busy;
+              pool = busy;
             }
         }
         if (this.getPoolProperties().isJmxEnabled()) {
-            this.jmxPool = null;
+          this.jmxPool = null;
         }
         PoolProperties.InterceptorDefinition[] proxies = getPoolProperties().getJdbcInterceptorsAsArray();
         for (int i=0; i<proxies.length; i++) {
@@ -430,7 +439,9 @@ public class ConnectionPool {
                 interceptor.setProperties(proxies[i].getProperties());
                 interceptor.poolClosed(this);
             }catch (Exception x) {
-                log.debug("Unable to inform interceptor of pool closure.",x);
+                if (log.isDebugEnabled()) {
+                    log.debug("Unable to inform interceptor of pool closure.",x);
+                }
             }
         }
     } //closePool
@@ -464,7 +475,7 @@ public class ConnectionPool {
 
         //create JMX MBean
         if (this.getPoolProperties().isJmxEnabled()) {
-            createMBean();
+          createMBean();
         }
 
         //Parse and create an initial set of interceptors. Letting them know the pool has started.
@@ -481,7 +492,7 @@ public class ConnectionPool {
             }catch (Exception x) {
                 log.error("Unable to inform interceptor of pool start.",x);
                 if (jmxPool!=null) {
-                    jmxPool.notify(org.apache.tomcat.jdbc.pool.jmx.ConnectionPool.NOTIFY_INIT, getStackTrace(x));
+                  jmxPool.notify(org.apache.tomcat.jdbc.pool.jmx.ConnectionPool.NOTIFY_INIT, getStackTrace(x));
                 }
                 close(true);
                 SQLException ex = new SQLException();
@@ -501,7 +512,7 @@ public class ConnectionPool {
             log.error("Unable to create initial connections of pool.", x);
             if (!poolProperties.isIgnoreExceptionOnPreLoad()) {
                 if (jmxPool!=null) {
-                    jmxPool.notify(org.apache.tomcat.jdbc.pool.jmx.ConnectionPool.NOTIFY_INIT, getStackTrace(x));
+                  jmxPool.notify(org.apache.tomcat.jdbc.pool.jmx.ConnectionPool.NOTIFY_INIT, getStackTrace(x));
                 }
                 close(true);
                 throw x;
@@ -540,6 +551,11 @@ public class ConnectionPool {
             log.warn("maxIdle is smaller than minIdle, setting maxIdle to: "+properties.getMinIdle());
             properties.setMaxIdle(properties.getMinIdle());
         }
+        if (properties.getMaxAge()>0 && properties.isPoolSweeperEnabled() &&
+                properties.getTimeBetweenEvictionRunsMillis()>properties.getMaxAge()) {
+            log.warn("timeBetweenEvictionRunsMillis is larger than maxAge, setting timeBetweenEvictionRunsMillis to: " + properties.getMaxAge());
+            properties.setTimeBetweenEvictionRunsMillis((int)properties.getMaxAge());
+        }
     }
 
     public void initializePoolCleaner(PoolConfiguration properties) {
@@ -570,7 +586,7 @@ public class ConnectionPool {
      */
     protected void abandon(PooledConnection con) {
         if (con == null) {
-            return;
+          return;
         }
         try {
             con.lock();
@@ -599,10 +615,10 @@ public class ConnectionPool {
      */
     protected void suspect(PooledConnection con) {
         if (con == null) {
-            return;
+          return;
         }
         if (con.isSuspect()) {
-            return;
+          return;
         }
         try {
             con.lock();
@@ -625,7 +641,7 @@ public class ConnectionPool {
      */
     protected void release(PooledConnection con) {
         if (con == null) {
-            return;
+          return;
         }
         try {
             con.lock();
@@ -675,7 +691,7 @@ public class ConnectionPool {
                 PooledConnection result = borrowConnection(now, con, username, password);
                 borrowedCount.incrementAndGet();
                 if (result!=null) {
-                    return result;
+                  return result;
                 }
             }
 
@@ -753,10 +769,10 @@ public class ConnectionPool {
         //no connections where available we'll create one
         PooledConnection con = create(false);
         if (username!=null) {
-            con.getAttributes().put(PooledConnection.PROP_USER, username);
+          con.getAttributes().put(PooledConnection.PROP_USER, username);
         }
         if (password!=null) {
-            con.getAttributes().put(PooledConnection.PROP_PASSWORD, password);
+          con.getAttributes().put(PooledConnection.PROP_PASSWORD, password);
         }
         boolean error = false;
         try {
@@ -793,7 +809,7 @@ public class ConnectionPool {
             }
         } finally {
             // con can never be null here
-            if (error ) {
+            if (error) {
                 release(con);
             }
             con.unlock();
@@ -851,10 +867,9 @@ public class ConnectionPool {
             try {
                 con.reconnect();
                 reconnectedCount.incrementAndGet();
-                int validationMode = getPoolProperties().isTestOnConnect() || getPoolProperties().getInitSQL()!=null ?
-                    PooledConnection.VALIDATE_INIT :
-                    PooledConnection.VALIDATE_BORROW;
-
+                int validationMode = isInitNewConnections() ?
+                        PooledConnection.VALIDATE_INIT:
+                        PooledConnection.VALIDATE_BORROW;
                 if (con.validate(validationMode)) {
                     //set the timestamp
                     con.setTimestamp(now);
@@ -888,6 +903,18 @@ public class ConnectionPool {
             }
         }
     }
+
+    /**
+     * Returns whether new connections should be initialized by invoking
+     * {@link PooledConnection#validate(int)} with {@link PooledConnection#VALIDATE_INIT}.
+     *
+     * @return true if pool is either configured to test connections on connect or a non-NULL init
+     * SQL has been configured
+     */
+    private boolean isInitNewConnections() {
+        return getPoolProperties().isTestOnConnect() || getPoolProperties().getInitSQL()!=null;
+    }
+
     /**
      * Terminate the current transaction for the given connection.
      * @param con The connection
@@ -900,12 +927,12 @@ public class ConnectionPool {
                 if (this.getPoolProperties().getRollbackOnReturn()) {
                     boolean autocommit = con.getConnection().getAutoCommit();
                     if (!autocommit) {
-                        con.getConnection().rollback();
+                      con.getConnection().rollback();
                     }
                 } else if (this.getPoolProperties().getCommitOnReturn()) {
                     boolean autocommit = con.getConnection().getAutoCommit();
                     if (!autocommit) {
-                        con.getConnection().commit();
+                      con.getConnection().commit();
                     }
                 }
             }
@@ -925,25 +952,48 @@ public class ConnectionPool {
      */
     protected boolean shouldClose(PooledConnection con, int action) {
         if (con.getConnectionVersion() < getPoolVersion()) {
-            return true;
+          return true;
         }
         if (con.isDiscarded()) {
-            return true;
+          return true;
         }
         if (isClosed()) {
-            return true;
+          return true;
         }
         if (!con.validate(action)) {
-            return true;
+          return true;
         }
         if (!terminateTransaction(con)) {
-            return true;
+          return true;
         }
+        return false;
+    }
+
+    /**
+     * Checks whether this connection has {@link PooledConnection#isMaxAgeExpired() expired} and tries to reconnect if it has.
+     * @param con PooledConnection
+     * @return true if the connection was either not expired or expired but reconnecting succeeded,
+     * false if reconnecting failed (either because a new connection could not be established or
+     * validating the newly created connection failed)
+     * @see PooledConnection#isMaxAgeExpired()
+     */
+    protected boolean reconnectIfExpired(PooledConnection con) {
         if (con.isMaxAgeExpired()) {
-            return true;
-        } else {
-            return false;
+            try {
+                if (log.isDebugEnabled()) {
+                  log.debug( "Connection ["+this+"] expired because of maxAge, trying to reconnect" );
+                }
+                con.reconnect();
+                reconnectedCount.incrementAndGet();
+                if ( isInitNewConnections() && !con.validate( PooledConnection.VALIDATE_INIT)) {
+                    return false;
+                }
+            } catch(Exception e) {
+                log.error("Failed to re-connect connection ["+this+"] that expired because of maxAge",e);
+                return false;
+            }
         }
+        return true;
     }
 
     /**
@@ -977,7 +1027,7 @@ public class ConnectionPool {
                 }
                 if (busy.remove(con)) {
 
-                    if (!shouldClose(con,PooledConnection.VALIDATE_RETURN)) {
+                    if (!shouldClose(con,PooledConnection.VALIDATE_RETURN) && reconnectIfExpired(con)) {
                         con.clearWarnings();
                         con.setStackTrace(null);
                         con.setTimestamp(System.currentTimeMillis());
@@ -1012,10 +1062,10 @@ public class ConnectionPool {
      */
     protected boolean shouldAbandon() {
         if (!poolProperties.isRemoveAbandoned()) {
-            return false;
+          return false;
         }
         if (poolProperties.getAbandonWhenPercentageFull()==0) {
-            return true;
+          return true;
         }
         float used = busy.size();
         float max  = poolProperties.getMaxActive();
@@ -1029,7 +1079,7 @@ public class ConnectionPool {
     public void checkAbandoned() {
         try {
             if (busy.isEmpty()) {
-                return;
+              return;
             }
             Iterator<PooledConnection> locked = busy.iterator();
             int sto = getPoolProperties().getSuspectTimeout();
@@ -1041,7 +1091,7 @@ public class ConnectionPool {
                     //the con has been returned to the pool or released
                     //ignore it
                     if (idle.contains(con) || con.isReleased()) {
-                        continue;
+                      continue;
                     }
                     long time = con.getTimestamp();
                     long now = System.currentTimeMillis();
@@ -1057,12 +1107,14 @@ public class ConnectionPool {
                 } finally {
                     con.unlock();
                     if (setToNull) {
-                        con = null;
+                      con = null;
                     }
                 }
             } //while
         } catch (ConcurrentModificationException e) {
-            log.debug("checkAbandoned failed." ,e);
+            if (log.isDebugEnabled()) {
+                log.debug("checkAbandoned failed." ,e);
+            }
         } catch (Exception e) {
             log.warn("checkAbandoned failed, it will be retried.",e);
         }
@@ -1080,7 +1132,7 @@ public class ConnectionPool {
 
         try {
             if (idle.isEmpty()) {
-                return;
+              return;
             }
             long now = System.currentTimeMillis();
             Iterator<PooledConnection> unlocked = idle.iterator();
@@ -1091,7 +1143,7 @@ public class ConnectionPool {
                     con.lock();
                     //the con been taken out, we can't clean it up
                     if (busy.contains(con)) {
-                        continue;
+                      continue;
                     }
                     long time = con.getTimestamp();
                     if (shouldReleaseIdle(now, con, time)) {
@@ -1105,12 +1157,14 @@ public class ConnectionPool {
                 } finally {
                     con.unlock();
                     if (setToNull) {
-                        con = null;
+                      con = null;
                     }
                 }
             } //while
         } catch (ConcurrentModificationException e) {
-            log.debug("checkIdle failed." ,e);
+            if (log.isDebugEnabled()) {
+                log.debug("checkIdle failed." ,e);
+            }
         } catch (Exception e) {
             log.warn("checkIdle failed, it will be retried.",e);
         }
@@ -1120,9 +1174,9 @@ public class ConnectionPool {
 
     protected boolean shouldReleaseIdle(long now, PooledConnection con, long time) {
         if (con.getConnectionVersion() < getPoolVersion()) {
-            return true;
+          return true;
         } else {
-            return (con.getReleaseTime()>0) && ((now - time) > con.getReleaseTime()) && (getSize()>getPoolProperties().getMinIdle());
+          return (con.getReleaseTime()>0) && ((now - time) > con.getReleaseTime()) && (getSize()>getPoolProperties().getMinIdle());
         }
     }
 
@@ -1130,9 +1184,18 @@ public class ConnectionPool {
      * Forces a validation of all idle connections if {@link PoolProperties#testWhileIdle} is set.
      */
     public void testAllIdle() {
+        testAllIdle(false);
+    }
+
+    /**
+     * Forces a validation of all idle connections if {@link PoolProperties#testWhileIdle} is set.
+     * @param checkMaxAgeOnly whether to only check {@link PooledConnection#isMaxAgeExpired()} but
+     *                        not invoke {@link PooledConnection#validate(int)}
+     */
+    public void testAllIdle(boolean checkMaxAgeOnly) {
         try {
             if (idle.isEmpty()) {
-                return;
+              return;
             }
             Iterator<PooledConnection> unlocked = idle.iterator();
             while (unlocked.hasNext()) {
@@ -1141,9 +1204,16 @@ public class ConnectionPool {
                     con.lock();
                     //the con been taken out, we can't clean it up
                     if (busy.contains(con)) {
-                        continue;
+                      continue;
                     }
-                    if (!con.validate(PooledConnection.VALIDATE_IDLE)) {
+
+                    boolean release;
+                    if (checkMaxAgeOnly) {
+                        release = !reconnectIfExpired(con);
+                    } else {
+                        release = !reconnectIfExpired(con) || !con.validate(PooledConnection.VALIDATE_IDLE);
+                    }
+                    if (release) {
                         releasedIdleCount.incrementAndGet();
                         unlocked.remove();
                         release(con);
@@ -1153,7 +1223,9 @@ public class ConnectionPool {
                 }
             } //while
         } catch (ConcurrentModificationException e) {
-            log.debug("testAllIdle failed." ,e);
+            if (log.isDebugEnabled()) {
+                log.debug("testAllIdle failed." ,e);
+            }
         } catch (Exception e) {
             log.warn("testAllIdle failed, it will be retried.",e);
         }
@@ -1196,7 +1268,7 @@ public class ConnectionPool {
      */
     protected PooledConnection create(boolean incrementCounter) {
         if (incrementCounter) {
-            size.incrementAndGet();
+          size.incrementAndGet();
         }
         PooledConnection con = new PooledConnection(getPoolProperties(), this);
         return con;
@@ -1395,7 +1467,7 @@ public class ConnectionPool {
             PooledConnection pc = this.pc!=null?this.pc:pcFuture.get(timeout,unit);
             if (pc!=null) {
                 if (result!=null) {
-                    return result;
+                  return result;
                 }
                 if (configured.compareAndSet(false, true)) {
                     try {
@@ -1413,7 +1485,7 @@ public class ConnectionPool {
                     latch.await(timeout,unit); //this shouldn't block for long
                 }
                 if (result==null) {
-                    throw new ExecutionException(cause);
+                  throw new ExecutionException(cause);
                 }
                 return result;
             } else {
@@ -1459,7 +1531,7 @@ public class ConnectionPool {
 
 
     private static volatile Timer poolCleanTimer = null;
-    private static HashSet<PoolCleaner> cleaners = new HashSet<>();
+    private static Set<PoolCleaner> cleaners = new HashSet<>();
 
     private static synchronized void registerCleaner(PoolCleaner cleaner) {
         unregisterCleaner(cleaner);
@@ -1540,17 +1612,19 @@ public class ConnectionPool {
                 try {
                     if (pool.getPoolProperties().isRemoveAbandoned()
                             || pool.getPoolProperties().getSuspectTimeout() > 0) {
-                        pool.checkAbandoned();
+                      pool.checkAbandoned();
                     }
                     if (pool.getPoolProperties().getMinIdle() < pool.idle
                             .size()) {
-                        pool.checkIdle();
+                      pool.checkIdle();
                     }
                     if (pool.getPoolProperties().isTestWhileIdle()) {
-                        pool.testAllIdle();
+                        pool.testAllIdle(false);
+                    } else if (pool.getPoolProperties().getMaxAge() > 0) {
+                        pool.testAllIdle(true);
                     }
                 } catch (Exception x) {
-                    log.error("", x);
+                    log.error(x.toString(), x);
                 }
             }
         }
