@@ -34,12 +34,13 @@ import org.apache.catalina.startup.SimpleHttpClient;
 import org.apache.catalina.startup.TesterServlet;
 import org.apache.catalina.startup.Tomcat;
 import org.apache.catalina.startup.TomcatBaseTest;
+import org.apache.tomcat.util.http.Method;
 
 public class TestHttp11InputBuffer extends TomcatBaseTest {
 
     private static final String CR = "\r";
     private static final String LF = "\n";
-    private  static final String CRLF = CR + LF;
+    private static final String CRLF = CR + LF;
 
     /**
      * Test case for https://bz.apache.org/bugzilla/show_bug.cgi?id=48839
@@ -76,13 +77,15 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
                 connect();
 
                 String[] request = new String[1];
+                // @formatter:off
                 request[0] =
-                    "GET http://localhost:8080/test HTTP/1.1" + CRLF +
-                    "Host: localhost:8080" + CRLF +
-                    "X-Bug48839: abcd" + CRLF +
-                    "\tefgh" + CRLF +
-                    "Connection: close" + CRLF +
-                    CRLF;
+                        "GET http://localhost:8080/test HTTP/1.1" + CRLF +
+                        "Host: localhost:8080" + CRLF +
+                        "X-Bug48839: abcd" + CRLF +
+                        "\tefgh" + CRLF +
+                        "Connection: close" + CRLF +
+                        CRLF;
+                // @formatter:on
 
                 setRequest(request);
                 processRequest(); // blocks until response has been read
@@ -116,8 +119,7 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
          * Only interested in the request headers from a GET request
          */
         @Override
-        protected void doGet(HttpServletRequest req, HttpServletResponse resp)
-                throws ServletException, IOException {
+        protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
             // Just echo the header value back as plain text
             resp.setContentType("text/plain");
 
@@ -167,9 +169,8 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
 
     @Test
     public void testBug51557SeparatorsInName() throws Exception {
-        char httpSeparators[] = new char[] {
-                '\t', ' ', '\"', '(', ')', ',', '/', ':', ';', '<',
-                '=', '>', '?', '@', '[', '\\', ']', '{', '}' };
+        char httpSeparators[] = new char[] { '\t', ' ', '\"', '(', ')', ',', '/', ':', ';', '<', '=', '>', '?', '@',
+                '[', '\\', ']', '{', '}' };
 
         for (char s : httpSeparators) {
             doTestBug51557CharInName(s);
@@ -222,8 +223,7 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
     @Test
     public void testBug51557Continuation() {
 
-        Bug51557Client client = new Bug51557Client("X-Bug=51557NoColon",
-                "foo" + SimpleHttpClient.CRLF + " bar");
+        Bug51557Client client = new Bug51557Client("X-Bug=51557NoColon", "foo" + SimpleHttpClient.CRLF + " bar");
 
         client.doRequest();
         Assert.assertTrue(client.isResponse200());
@@ -235,8 +235,7 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
     @Test
     public void testBug51557BoundaryStart() {
 
-        Bug51557Client client = new Bug51557Client("=X-Bug51557",
-                "invalid");
+        Bug51557Client client = new Bug51557Client("=X-Bug51557", "invalid");
 
         client.doRequest();
         Assert.assertTrue(client.isResponse200());
@@ -248,8 +247,7 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
     @Test
     public void testBug51557BoundaryEnd() {
 
-        Bug51557Client client = new Bug51557Client("X-Bug51557=",
-                "invalid");
+        Bug51557Client client = new Bug51557Client("X-Bug51557=", "invalid");
 
         client.doRequest();
         Assert.assertTrue(client.isResponse200());
@@ -259,8 +257,7 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
 
 
     private void doTestBug51557CharInName(char s) {
-        Bug51557Client client =
-            new Bug51557Client("X-Bug" + s + "51557", "invalid");
+        Bug51557Client client = new Bug51557Client("X-Bug" + s + "51557", "invalid");
 
         client.doRequest();
         Assert.assertTrue(client.isResponse200());
@@ -270,8 +267,7 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
 
 
     private void doTestBug51557InvalidCharInValue(char s) {
-        Bug51557Client client =
-            new Bug51557Client("X-Bug51557-Invalid", "invalid" + s + "invalid");
+        Bug51557Client client = new Bug51557Client("X-Bug51557-Invalid", "invalid" + s + "invalid");
 
         client.doRequest();
         Assert.assertTrue("Testing [" + (int) s + "]", client.isResponse200());
@@ -281,8 +277,7 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
 
 
     private void doTestBug51557ValidCharInValue(char s) {
-        Bug51557Client client =
-            new Bug51557Client("X-Bug51557-Valid", "valid" + s + "valid");
+        Bug51557Client client = new Bug51557Client("X-Bug51557-Valid", "valid" + s + "valid");
 
         client.doRequest();
         Assert.assertTrue("Testing [" + (int) s + "]", client.isResponse200());
@@ -300,18 +295,17 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
         private final String headerLine;
         private final boolean rejectIllegalHeader;
 
-        public Bug51557Client(String headerName) {
+        Bug51557Client(String headerName) {
             this.headerName = headerName;
             this.headerLine = headerName;
             this.rejectIllegalHeader = false;
         }
 
-        public Bug51557Client(String headerName, String headerValue) {
+        Bug51557Client(String headerName, String headerValue) {
             this(headerName, headerValue, false);
         }
 
-        public Bug51557Client(String headerName, String headerValue,
-                boolean rejectIllegalHeader) {
+        Bug51557Client(String headerName, String headerValue, boolean rejectIllegalHeader) {
             this.headerName = headerName;
             this.headerLine = headerName + ": " + headerValue;
             this.rejectIllegalHeader = rejectIllegalHeader;
@@ -322,14 +316,12 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
             Tomcat tomcat = getTomcatInstance();
 
             Context root = tomcat.addContext("", TEMP_DIR);
-            Tomcat.addServlet(root, "Bug51557",
-                    new Bug51557Servlet(headerName));
+            Tomcat.addServlet(root, "Bug51557", new Bug51557Servlet(headerName));
             root.addServletMappingDecoded("/test", "Bug51557");
 
             try {
                 Connector connector = tomcat.getConnector();
-                Assert.assertTrue(connector.setProperty(
-                        "rejectIllegalHeader", Boolean.toString(rejectIllegalHeader)));
+                Assert.assertTrue(connector.setProperty("rejectIllegalHeader", Boolean.toString(rejectIllegalHeader)));
                 tomcat.start();
                 setPort(connector.getLocalPort());
 
@@ -338,13 +330,15 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
                 connect();
 
                 String[] request = new String[1];
+                // @formatter:off
                 request[0] =
-                    "GET http://localhost:8080/test HTTP/1.1" + CRLF +
-                    "Host: localhost:8080" + CRLF +
-                    headerLine + CRLF +
-                    "X-Bug51557: abcd" + CRLF +
-                    "Connection: close" + CRLF +
-                    CRLF;
+                        "GET http://localhost:8080/test HTTP/1.1" + CRLF +
+                        "Host: localhost:8080" + CRLF +
+                        headerLine + CRLF +
+                        "X-Bug51557: abcd" + CRLF +
+                        "Connection: close" + CRLF +
+                        CRLF;
+                // @formatter:on
 
                 setRequest(request);
                 processRequest(); // blocks until response has been read
@@ -377,10 +371,9 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
         private String invalidHeaderName;
 
         /**
-         * @param invalidHeaderName The header name should be invalid and
-         *                          therefore ignored by the header parsing code
+         * @param invalidHeaderName The header name should be invalid and therefore ignored by the header parsing code
          */
-        public Bug51557Servlet(String invalidHeaderName) {
+        Bug51557Servlet(String invalidHeaderName) {
             this.invalidHeaderName = invalidHeaderName;
         }
 
@@ -388,8 +381,7 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
          * Only interested in the request headers from a GET request
          */
         @Override
-        protected void doGet(HttpServletRequest req, HttpServletResponse resp)
-                throws ServletException, IOException {
+        protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
             // Just echo the header value back as plain text
             resp.setContentType("text/plain");
 
@@ -399,19 +391,18 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
             processHeaders("X-Bug51557", req, out);
         }
 
-        private void processHeaders(String header, HttpServletRequest req,
-                PrintWriter out) {
+        private void processHeaders(String header, HttpServletRequest req, PrintWriter out) {
             Enumeration<String> values = req.getHeaders(header);
             while (values.hasMoreElements()) {
-                out.println(values.nextElement());
+                out.print(values.nextElement());
             }
         }
     }
 
 
     /**
-     * Test case for new lines at the start of a request. RFC2616
-     * does not permit any, but Tomcat is tolerant of them if they are present.
+     * Test case for new lines at the start of a request. RFC2616 does not permit any, but Tomcat is tolerant of them if
+     * they are present.
      */
     @Test
     public void testNewLines() {
@@ -425,8 +416,8 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
 
 
     /**
-     * Test case for new lines at the start of a request. RFC2616
-     * does not permit any, but Tomcat is tolerant of them if they are present.
+     * Test case for new lines at the start of a request. RFC2616 does not permit any, but Tomcat is tolerant of them if
+     * they are present.
      */
     @Test
     public void testNewLinesExcessive() {
@@ -471,14 +462,16 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
                 connect();
 
                 String[] request = new String[1];
+                // @formatter:off
                 request[0] =
-                    newLines +
-                    "GET http://localhost:8080/test HTTP/1.1" + CRLF +
-                    "Host: localhost:8080" + CRLF +
-                    "X-Bug48839: abcd" + CRLF +
-                    "\tefgh" + CRLF +
-                    "Connection: close" + CRLF +
-                    CRLF;
+                        newLines +
+                        "GET http://localhost:8080/test HTTP/1.1" + CRLF +
+                        "Host: localhost:8080" + CRLF +
+                        "X-Bug48839: abcd" + CRLF +
+                        "\tefgh" + CRLF +
+                        "Connection: close" + CRLF +
+                        CRLF;
+                // @formatter:on
 
                 setRequest(request);
                 processRequest(); // blocks until response has been read
@@ -541,10 +534,7 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
 
                 String[] request = new String[2];
                 request[0] = "GET http://localhost:8080/test HTTP/1.1" + CR;
-                request[1] = LF +
-                        "Host: localhost:8080" + CRLF +
-                        "Connection: close" + CRLF +
-                        CRLF;
+                request[1] = LF + "Host: localhost:8080" + CRLF + "Connection: close" + CRLF + CRLF;
 
                 setRequest(request);
                 processRequest(); // blocks until response has been read
@@ -611,10 +601,8 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
                 connect();
 
                 String[] request = new String[1];
-                request[0] = "GET http://localhost:8080/test HTTP/1.1" + CRLF +
-                        "Host: localhost:8080" + CRLF +
-                        "X-Header: Ignore" + CRLF +
-                        "X-Header" + (char) 130 + ": Broken" + CRLF + CRLF;
+                request[0] = "GET http://localhost:8080/test HTTP/1.1" + CRLF + "Host: localhost:8080" + CRLF +
+                        "X-Header: Ignore" + CRLF + "X-Header" + (char) 130 + ": Broken" + CRLF + CRLF;
 
                 setRequest(request);
                 processRequest(); // blocks until response has been read
@@ -644,11 +632,8 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
     public void testInvalidMethod() {
 
         String[] request = new String[1];
-        request[0] =
-            "GET" + (char) 0 + " /test HTTP/1.1" + CRLF +
-            "Host: localhost:8080" + CRLF +
-            "Connection: close" + CRLF +
-            CRLF;
+        request[0] = Method.GET + (char) 0 + " /test HTTP/1.1" + CRLF + "Host: localhost:8080" + CRLF + "Connection: close" +
+                CRLF + CRLF;
 
         InvalidClient client = new InvalidClient(request);
 
@@ -676,11 +661,7 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
     public void testInvalidEndOfRequestLine01() {
 
         String[] request = new String[1];
-        request[0] =
-                "GET /test HTTP/1.1" + CR +
-                "Host: localhost:8080" + CRLF +
-                "Connection: close" + CRLF +
-                CRLF;
+        request[0] = "GET /test HTTP/1.1" + CR + "Host: localhost:8080" + CRLF + "Connection: close" + CRLF + CRLF;
 
         InvalidClient client = new InvalidClient(request);
 
@@ -694,12 +675,8 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
     public void testInvalidHeader01() {
 
         String[] request = new String[1];
-        request[0] =
-                "GET /test HTTP/1.1" + CRLF +
-                "Host: localhost:8080" + CRLF +
-                CR + "X-Header: xxx" + CRLF +
-                "Connection: close" + CRLF +
-                CRLF;
+        request[0] = "GET /test HTTP/1.1" + CRLF + "Host: localhost:8080" + CRLF + CR + "X-Header: xxx" + CRLF +
+                "Connection: close" + CRLF + CRLF;
 
         InvalidClient client = new InvalidClient(request);
 
@@ -725,12 +702,8 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
         getTomcatInstance().getConnector().setProperty("rejectIllegalHeader", Boolean.toString(rejectIllegalHeader));
 
         String[] request = new String[1];
-        request[0] =
-                "POST /test HTTP/1.1" + CRLF +
-                "Host: localhost:8080" + CRLF +
-                "Content-Length: 12\u000734" + CRLF +
-                "Connection: close" + CRLF +
-                CRLF;
+        request[0] = "POST /test HTTP/1.1" + CRLF + "Host: localhost:8080" + CRLF + "Content-Length: 12\u000734" +
+                CRLF + "Connection: close" + CRLF + CRLF;
 
         InvalidClient client = new InvalidClient(request);
 
@@ -747,7 +720,7 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
 
         private final String[] request;
 
-        public InvalidClient(String[] request) {
+        InvalidClient(String[] request) {
             this.request = request;
         }
 
@@ -782,4 +755,54 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
             return true;
         }
     }
+
+
+    private static final class Client extends SimpleHttpClient {
+
+        Client(int port) {
+            setPort(port);
+        }
+
+        @Override
+        public boolean isResponseBodyOK() {
+            return getResponseBody().contains("test - data");
+        }
+    }
+
+
+    @Test
+    public void testInvalidHeader02() throws Exception {
+        Tomcat tomcat = getTomcatInstance();
+
+        // This setting means the connection will be closed at the end of the
+        // request
+        Assert.assertTrue(tomcat.getConnector().setProperty("maxKeepAliveRequests", "1"));
+
+        // No file system docBase required
+        Context ctx = getProgrammaticRootContext();
+
+        // Add servlet
+        Tomcat.addServlet(ctx, "TesterServlet", new TesterServlet());
+        ctx.addServletMappingDecoded("/foo", "TesterServlet");
+
+        tomcat.start();
+
+        // @formatter:off
+        String request =
+                "GET /foo HTTP/1.1" + SimpleHttpClient.CRLF +
+                "Host: localhost" + SimpleHttpClient.CRLF +
+                ":b" + SimpleHttpClient.CRLF +
+                "X-Dummy:b" + SimpleHttpClient.CRLF +
+                SimpleHttpClient.CRLF;
+        // @formatter:on
+
+        Client client = new Client(tomcat.getConnector().getLocalPort());
+        client.setRequest(new String[] { request });
+
+        client.connect(10000, 600000);
+        client.processRequest();
+
+        // Expected response is a 400 response.
+        Assert.assertTrue(client.getResponseLine(), client.isResponse400());
+    }
 }
