@@ -19,6 +19,7 @@ package org.apache.tomcat.util.net.ocsp;
 
 import java.io.Closeable;
 import java.io.File;
+import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.net.InetSocketAddress;
@@ -26,7 +27,6 @@ import java.net.ServerSocket;
 import java.net.URI;
 import java.net.URL;
 import java.nio.file.Files;
-import java.nio.file.Paths;
 import java.security.KeyStore;
 import java.security.KeyStoreException;
 import java.security.cert.CRLReason;
@@ -57,7 +57,8 @@ import javax.net.ssl.SSLHandshakeException;
 import javax.net.ssl.SSLSocketFactory;
 import javax.net.ssl.TrustManagerFactory;
 import javax.security.auth.x500.X500Principal;
-import javax.servlet.http.HttpServletResponse;
+
+import jakarta.servlet.http.HttpServletResponse;
 
 import org.junit.Assert;
 import org.junit.Assume;
@@ -105,7 +106,7 @@ public class TestOcspIntegration extends TomcatBaseTest {
                 "JSSE", Boolean.FALSE, "org.apache.tomcat.util.net.jsse.JSSEImplementation"});
          */
         /*
-         * Disabled until a Tomcat Native release (1.3.2 onwards) is available with the OCSP fix
+         * Disabled until a Tomcat Native release (2.0.10 onwards) is available with the OCSP fix
         parameterSets.add(new Object[] {
                 "OpenSSL", Boolean.TRUE, "org.apache.tomcat.util.net.openssl.OpenSSLImplementation"});
         */
@@ -223,11 +224,13 @@ public class TestOcspIntegration extends TomcatBaseTest {
         context.addServletMappingDecoded("/", "simple");
 
         KeyStore trustStore = KeyStore.getInstance(KEYSTORE_TYPE);
-        String trustStorePass = new String(Files.readAllBytes(new File(getPath(TRUSTSTORE_PASS)).toPath())).trim();
-        trustStore.load(Files.newInputStream(Paths.get(new File(getPath(TRUSTSTORE_PATH)).getAbsolutePath())), trustStorePass.toCharArray());
+        String trustStorePass = Files.readString(new File(getPath(TRUSTSTORE_PASS)).toPath()).trim();
+        trustStore.load(new FileInputStream(new File(getPath(TRUSTSTORE_PATH)).getAbsolutePath()),
+                trustStorePass.toCharArray());
         KeyStore clientKeystore = KeyStore.getInstance(KEYSTORE_TYPE);
-        String clientKeystorePass = new String(Files.readAllBytes(new File(getPath(CLIENT_KEYSTORE_PASS)).toPath())).trim();
-        clientKeystore.load(Files.newInputStream(Paths.get(new File(getPath(CLIENT_KEYSTORE_PATH)).getAbsolutePath())), clientKeystorePass.toCharArray());
+        String clientKeystorePass = Files.readString(new File(getPath(CLIENT_KEYSTORE_PASS)).toPath()).trim();
+        clientKeystore.load(new FileInputStream(new File(getPath(CLIENT_KEYSTORE_PATH)).getAbsolutePath()),
+                clientKeystorePass.toCharArray());
         byte[] ocspResponse = Files.readAllBytes(new File(getPath(pathToOcspResponse)).toPath());
         try (FakeOcspResponder fakeOcspResponder = new FakeOcspResponder(ocspResponse, ocspResponderPort)) {
             fakeOcspResponder.start();
@@ -378,7 +381,7 @@ public class TestOcspIntegration extends TomcatBaseTest {
         if (file == null) {
             return null;
         }
-        String packageName = this.getClass().getPackage().getName();
+        String packageName = this.getClass().getPackageName();
         String path = packageName.replace(".", File.separator);
         File f = new File("test" + File.separator + path + File.separator + file);
 
